package index.MyMtree;

import java.util.Arrays;

public final class SSTreeSplitHandler {
	private Sphere toSplit;
	
	public void split(Sphere splitMe) {
		this.toSplit = splitMe;
		
		//(1) Find the split dimension -> That one having highest variance
		int variances[] = new int[toSplit.center.length];
		int indexHighestVariance = 0;
		int highestVariance;
		variances[0] = highestVariance = getVariance(0);
		
		for(int dim=1;dim<toSplit.center.length;dim++){//starting with 1
			variances[dim] = getVariance(dim);
			if(variances[dim]>highestVariance){
				highestVariance = variances[dim];
				indexHighestVariance = dim;
			}
		}
		final int SPLIT_DIMENSION = indexHighestVariance;
		
		//(2) Now distribute the points by minimizing the variance of the resulting two spheres
		distribute(SPLIT_DIMENSION);
	}

	private void distribute(final int SPLIT_DIMENSION) {
		MyContainer[] temp = new MyContainer[toSplit.size()]; 
		final int SPLIT_SIZE = index.structures.IndexMyMtree.tree.SPLIT_SIZE;
		
		//////////////////Clone getVariance()/////////////////
		int avg = 0;
		final int SIZE = toSplit.size();
		int localCoordinate;
		
		//(1) Dimension average
		for (int child=0;child<SIZE;child++){
			localCoordinate = toSplit.children[child].center[SPLIT_DIMENSION];
			avg += localCoordinate; 
		}
		avg /= SIZE;
		
		//(2) Now the variance
		int dist2avg = 0;
		//sum variance for all spheres
		for (int child=0;child<SIZE;child++){
			localCoordinate = toSplit.children[child].center[SPLIT_DIMENSION];
			dist2avg = avg-localCoordinate;
			temp[child] = new MyContainer(dist2avg, toSplit.children[child]);
		}
		//////////////////Clone getVariance/////////////////
		
		Arrays.sort(temp);
		Sphere parent = (toSplit.isRoot())? toSplit : toSplit.parent;
		Sphere lower = new Sphere(SPLIT_SIZE, parent);
		Sphere upper = new Sphere(SPLIT_SIZE, parent);
		final int BORDER = SIZE/2;
		
		//fill lower Sphere
		for(int child = 0; child<BORDER;child++){
			Sphere s = temp[child].SPHERE;
			lower.children[child] = s;
			s.parent = lower;
			lower.size++;
		}
		lower.computeCenterAndRadius();
		
		//fill upper Sphere
		for(int child = 0; child+BORDER<temp.length;child++){
			Sphere s = temp[child+BORDER].SPHERE;
			upper.children[child] = s;
			s.parent = upper;
			upper.size++;
		}
		upper.computeCenterAndRadius();
	
		// adjust parent
		handleLinkingSpheres(lower, upper);
		if(parent.size() == SPLIT_SIZE){
			split(toSplit.parent);
		}else{
			parent.checkForAdjust();
		}
		index.MyMtree.Test.validate();
	}

	void handleLinkingSpheres(Sphere lower, Sphere upper){
		if(toSplit.isRoot()){
			toSplit.children[0] = lower;
			toSplit.children[1] = upper;
			for(int i = 2; i<toSplit.size();i++){
				toSplit.children[i] = null;//just for me
			}
			toSplit.size = 2;
			toSplit.computeCenterAndRadius();
		}else{
			//get index of this, in parent's children
			int i=0;
			final Sphere PARENT = toSplit.parent;
			
			for (;i<PARENT.size();i++){
				if (PARENT.children[i]==toSplit){
					PARENT.children[i] = lower;
					break;
				}
			}
			PARENT.children[PARENT.size()] = upper;
			PARENT.size++;
		}
	}
	
	private int getVariance(final int dim) {
		int avg = 0;
		final int SIZE = toSplit.size();
		int localCoordinate;
		
		//(1) Dimension average
		for (int child=0;child<SIZE;child++){
			localCoordinate = toSplit.children[child].center[dim];
			avg += localCoordinate; 
		}
		avg /= SIZE;
		
		//(2) Now the variance
		int variance = 0;
		//sum variance for all spheres
		for (int child=0;child<SIZE;child++){
			localCoordinate = toSplit.children[child].center[dim];
			variance += ((avg-localCoordinate)*(avg-localCoordinate));
		}
		return variance;
	}
	
	private final class MyContainer implements Comparable<MyContainer>{
		final int DIST2AVG;
		final Sphere SPHERE;
		
		MyContainer(final int v, final Sphere s){
			this.DIST2AVG 	= v;
			this.SPHERE		= s;
		}

		@Override
		public int compareTo(MyContainer o) {
			return (DIST2AVG<o.DIST2AVG) ? 1 : -1;
		}
	}
}
