package index.structures;

import java.util.HashMap;
import java.util.LinkedList;

import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JPanel;

import testapp.Dictionary;
import testapp.JOptionDialog;
import testapp.Dictionary.Bundle;
import testapp.tapedpane.JTappedPane_Panel;
import index.MyMtree.*;
import index.base.AIndexStructure;

public final class IndexMyMtree extends AIndexStructure {
	public static IndexMyMtree tree;
	private Sphere root;
	public int SPLIT_SIZE = 4;
	public final int DIM;
	
	public IndexMyMtree(int _size, int _dim, int _minValue, int _maxValue,
			HashMap<String, Object> _map) {
		super(_size, _dim, _minValue, _maxValue, _map);
		initParameter(_map);
		
		SPLIT_SIZE++;
		IndexMyMtree.tree = this;
		DIM = _dim;
		root = Sphere.getRoot();
	}
	
	private int tid=0;
	@Override
	protected boolean insert(int[] _dataset) {
		Sphere insertHere;
		insertHere = root.getLeaf(_dataset);
		insertHere.add(_dataset, tid++, SPLIT_SIZE); 
		return true;
	}

	@Override
	public int[] search(int[] _datasetToSearch) {
		return root.getExact(_datasetToSearch);
	}

	@Override
	public int[][] searchKNN(int[] _datasetToSearch, int _k) {
		Neighbors nn = new Neighbors(_k, _datasetToSearch);
		root.getNN(nn);
		return nn.toArray();
	}

	@Override
	public boolean canKNNSearch() {
		return true;
	}

	@Override
	public String getInformations() {
		return "Mtree with max "+(SPLIT_SIZE-1)+" points per node";
	}

	@Override
	public String getName() {
		return "Mtree";
	}

	@Override
	public void clear() {
		this.root = null;
		System.gc();//kindly ask the garbage collector to start working ...
	}
	
	public Sphere getRoot(){return root;}

	
	String out(){//TODO
		String out = "";
		LinkedList<Sphere> nextLevel = new LinkedList<Sphere>();
		LinkedList<Sphere> thisLevel = new LinkedList<Sphere>();
		
		LinkedList<Sphere> nodes = new LinkedList<Sphere>();
		nodes.add(root);
		Sphere current;
		
		while(nodes.size()>0){
			current = nodes.removeFirst();
			out+="";
			for(int i=0;i<current.size();i++){
				if(!current.isPoint())
					nodes.add(current.children[i]);
			}
		}
		
		return out;
	}
	
	@Override
	protected JOptionDialog getOptionsDialog(JFrame _parent) {
		return new MOptionsDialog(_parent, this);
	}
	
	//////////////////////////////GUI stuff///////////////////////////
	
	private class MOptionsDialog extends JOptionDialog {
		private static final long serialVersionUID = 1L;
	
		JFormattedTextField fieldMax;
		JTappedPane_Panel northPanel;
	
		public MOptionsDialog(JFrame parent, AIndexStructure is) {
			super(parent, is.getName());
		}
	
		public JPanel getNorthPanel() {
			northPanel = new JTappedPane_Panel(null){
				private static final long serialVersionUID = 1L;
	
				@Override
				public void refreshResource() {
					
				}
				
			};
			
			fieldMax = northPanel.getFormtedTextFieldInstance_Integer(1);
			fieldMax.setValue(SPLIT_SIZE);
			fieldMax.setColumns(2);
	
			northPanel.addSimpleLabelAndInputRow(1, Dictionary.get(Bundle.INDEX,"maxpoints")+":", fieldMax);
	
			return northPanel;
		}
	
		@Override
		public HashMap<String, Object> getOptions() {
			HashMap<String, Object> map = new HashMap<String, Object>();
			map.put("SPLIT_SIZE", "" + ((Number)fieldMax.getValue()).intValue());
			return map;
		}
	}
}
